@@ -7,22 +7,30 @@ Defines MainWindow class.
 """
 
 from pathlib import Path
-from typing import List, Optional
-from PyQt5 import QtWidgets, QtCore
+from typing import Optional, List
+from PyQt6 import QtWidgets
+from PyQt6.QtCore import Qt, QTimer
+from PyQt6.QtWidgets import QMainWindow, QSizePolicy, QApplication
 
-from . import APP_TITLE
-from .mda_folder import MDA_MVC
-from . import utils
-from .user_settings import settings
-from .opendialog import DIR_SETTINGS_KEY
-from .lazy_folder_scanner import LazyFolderScanner, FolderScanResult
+# QAction import removed - no longer needed
+
+from mdaviz import APP_TITLE
+from mdaviz.mda_folder import MDA_MVC
+from mdaviz import utils
+from mdaviz.user_settings import settings
+from mdaviz.opendialog import DIR_SETTINGS_KEY
+from mdaviz.lazy_folder_scanner import LazyFolderScanner, FolderScanResult
+from mdaviz.logger import get_logger
+
+# Get logger for this module
+logger = get_logger("mainwindow")
 
 UI_FILE = utils.getUiFileName(__file__)
 MAX_FILES = 500
 MAX_RECENT_DIRS = 10
 
 
-class MainWindow(QtWidgets.QMainWindow):
+class MainWindow(QMainWindow):
     """The main window of the app, built in Qt designer.
 
     .. autosummary::
@@ -56,24 +64,22 @@ class MainWindow(QtWidgets.QMainWindow):
 
         # Set proper window flags for macOS resizing
         self.setWindowFlags(
-            QtCore.Qt.Window
-            | QtCore.Qt.WindowMinimizeButtonHint
-            | QtCore.Qt.WindowMaximizeButtonHint
-            | QtCore.Qt.WindowCloseButtonHint
+            Qt.WindowType.Window
+            | Qt.WindowType.WindowMinimizeButtonHint
+            | Qt.WindowType.WindowMaximizeButtonHint
+            | Qt.WindowType.WindowCloseButtonHint
         )
 
         # Additional macOS-specific properties for proper resizing
-        self.setAttribute(QtCore.Qt.WA_MacShowFocusRect, False)
-        self.setAttribute(QtCore.Qt.WA_MacNormalSize, True)
+        self.setAttribute(Qt.WidgetAttribute.WA_MacShowFocusRect, False)
+        self.setAttribute(Qt.WidgetAttribute.WA_MacNormalSize, True)
 
         # Ensure the window can be resized
         self.setMinimumSize(400, 300)
         self.resize(720, 400)  # More reasonable initial size
 
         # Ensure the window is resizable
-        self.setSizePolicy(
-            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
-        )
+        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
 
         # Ensure central widget and main content area are resizable
         self._setup_resizable_layout()
@@ -95,28 +101,38 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.connect()
 
-        settings.restoreWindowGeometry(self, "mainwindow_geometry")
-        print("Settings are saved in:", settings.fileName())
+        # Restore window geometry from settings
+        geometry_restored = settings.restoreWindowGeometry(self, "mainwindow_geometry")
+        logger.info(f"Settings are saved in: {settings.fileName()}")
 
-        # Ensure the window size is reasonable (not too large)
-        screen = QtWidgets.QDesktopWidget().screenGeometry()
-        max_width = min(screen.width() * 0.8, 1200)  # Max 80% of screen width or 1200px
-        max_height = min(
-            screen.height() * 0.8, 800
-        )  # Max 80% of screen height or 800px
+        # Calculate screen size constraints
+        screen = QApplication.primaryScreen().geometry()
+        max_width = screen.width() * 0.8  # Max 80% of screen width
+        max_height = screen.height() * 0.8  # Max 80% of screen height
 
-        if self.width() > max_width or self.height() > max_height:
-            self.resize(
-                int(min(self.width(), max_width)), int(min(self.height(), max_height))
-            )
-        elif self.width() < 400 or self.height() < 300:
-            self.resize(720, 400)  # Reasonable default size
+        # Only apply size constraints and centering if no geometry was previously saved
+        if not geometry_restored:
+            # Ensure the window size is reasonable (not too large)
+            if self.width() > max_width or self.height() > max_height:
+                self.resize(
+                    int(min(self.width(), max_width)),
+                    int(min(self.height(), max_height)),
+                )
+            elif self.width() < 400 or self.height() < 300:
+                self.resize(720, 500)  # Reasonable default size
 
-        # Center the window on the screen
-        self._center_window()
+            # Center the window on the screen only if no geometry was restored
+            self._center_window()
+        else:
+            # If geometry was restored, only apply size constraints if window is unreasonably large
+            if self.width() > max_width or self.height() > max_height:
+                self.resize(
+                    int(min(self.width(), max_width)),
+                    int(min(self.height(), max_height)),
+                )
 
-        # Auto-load the first valid folder from recent folders
-        self._auto_load_first_folder()
+        # Auto-load the first valid folder from recent folders (delayed to preserve startup message)
+        QTimer.singleShot(100, self._auto_load_first_folder)
 
     def connect(self):
         self.actionOpen.triggered.connect(self.doOpen)
@@ -127,31 +143,12 @@ class MainWindow(QtWidgets.QMainWindow):
         utils.reconnect(self.refresh.released, self.onRefresh)
         self.folder.currentTextChanged.connect(self.onFolderSelected)
 
-        # Add auto-load toggle action to File menu
-        self._setup_auto_load_menu()
+        # Auto-load menu removed - now handled in preferences dialog
 
         # Create tab widget and fit data components if they don't exist
         self._setup_fit_data_tab()
 
-    def _setup_auto_load_menu(self):
-        """Set up the auto-load toggle menu action."""
-        # Create the auto-load toggle action
-        self.actionToggleAutoLoad = QtWidgets.QAction("Toggle Auto-Load", self)
-        self.actionToggleAutoLoad.setStatusTip(
-            "Toggle automatic folder loading on startup"
-        )
-        self.actionToggleAutoLoad.setCheckable(True)
-        self.actionToggleAutoLoad.setChecked(self.get_auto_load_setting())
-        self.actionToggleAutoLoad.triggered.connect(self._on_toggle_auto_load)
-
-        # Add to File menu
-        self.menuFile.addSeparator()
-        self.menuFile.addAction(self.actionToggleAutoLoad)
-
-    def _on_toggle_auto_load(self):
-        """Handle auto-load toggle menu action."""
-        new_setting = self.toggle_auto_load()
-        self.actionToggleAutoLoad.setChecked(new_setting)
+    # Auto-load menu methods removed - now handled in preferences dialog
 
     @property
     def status(self):
@@ -159,14 +156,17 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def setStatus(self, text, timeout=0):
         """Write new status to the main window and terminal output."""
-        print(text)
+        # Avoid logging duplicate consecutive messages
+        if not hasattr(self, "_last_status") or self._last_status != text:
+            logger.info(text)
+            self._last_status = text
         self.statusbar.showMessage(str(text), msecs=timeout)
 
     def doAboutDialog(self, *args, **kw):
         """
         Show the "About ..." dialog
         """
-        from .aboutdialog import AboutDialog
+        from mdaviz.aboutdialog import AboutDialog
 
         about = AboutDialog(self)
         about.open()
@@ -175,13 +175,13 @@ class MainWindow(QtWidgets.QMainWindow):
         """
         Show the Preferences dialog
         """
-        from PyQt5.QtWidgets import (
+        from PyQt6.QtWidgets import (
             QDialog,
             QVBoxLayout,
-            QHBoxLayout,
             QLabel,
             QSpinBox,
             QDialogButtonBox,
+            QCheckBox,
         )
 
         # Create preferences dialog
@@ -192,8 +192,20 @@ class MainWindow(QtWidgets.QMainWindow):
 
         layout = QVBoxLayout(dialog)
 
-        # Plot height setting
-        plot_layout = QHBoxLayout()
+        # Auto-load setting (first)
+        auto_load_checkbox = QCheckBox("Auto-load first folder on startup")
+        auto_load_val = settings.getKey("auto_load_folder")
+        if auto_load_val is None:
+            auto_load_val = True
+        elif isinstance(auto_load_val, str):
+            auto_load_val = auto_load_val.lower() in ("true", "1", "yes", "on")
+        auto_load_checkbox.setChecked(bool(auto_load_val))
+        layout.addWidget(auto_load_checkbox)
+
+        # Add some spacing
+        layout.addStretch()
+
+        # Plot height setting (second)
         plot_label = QLabel("Maximum Plot Height (pixels):")
         plot_spinbox = QSpinBox()
         plot_spinbox.setRange(200, 2000)
@@ -205,25 +217,38 @@ class MainWindow(QtWidgets.QMainWindow):
             plot_height_val = 800
         plot_spinbox.setValue(plot_height_val)
         plot_spinbox.setSuffix(" px")
-        plot_layout.addWidget(plot_label)
-        plot_layout.addWidget(plot_spinbox)
-        layout.addLayout(plot_layout)
+        layout.addWidget(plot_label)
+        layout.addWidget(plot_spinbox)
+
+        # Add helpful caption for plot height setting
+        plot_caption = QLabel(
+            "Use this setting if plot areas expand vertically unexpectedly due to UI bugs."
+        )
+        plot_caption.setWordWrap(True)
+        plot_caption.setStyleSheet("color: gray; font-size: 10px;")
+        layout.addWidget(plot_caption)
 
         # Add some spacing
         layout.addStretch()
 
         # Buttons
-        button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        button_box = QDialogButtonBox(
+            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
+        )
         button_box.accepted.connect(dialog.accept)
         button_box.rejected.connect(dialog.reject)
         layout.addWidget(button_box)
 
         # Show dialog
-        if dialog.exec_() == QDialog.Accepted:
+        if dialog.exec() == QDialog.DialogCode.Accepted:
             # Save the new plot height setting
             new_height = plot_spinbox.value()
             settings.setKey("plot_max_height", new_height)
 
+            # Save the new auto-load setting
+            new_auto_load = auto_load_checkbox.isChecked()
+            settings.setKey("auto_load_folder", new_auto_load)
+
             # Update any existing ChartView widgets
             if hasattr(self, "mvc_folder") and self.mvc_folder:
                 if (
@@ -244,7 +269,9 @@ class MainWindow(QtWidgets.QMainWindow):
                     ):
                         self.mvc_folder.mda_file_viz._updateTabWidgetMaxHeight()
 
-            self.setStatus(f"Plot height setting updated to {new_height} pixels")
+            self.setStatus(
+                f"Settings updated: plot height {new_height}px, auto-load {'enabled' if new_auto_load else 'disabled'}"
+            )
 
     def closeEvent(self, event):
         """
@@ -268,16 +295,17 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def doOpen(self, *args, **kw):
         """
-        User chose to open (connect with) a tiled server.
+        User chose to open a file or folder dialog.
         """
-        from .opendialog import OpenDialog
+        from mdaviz.opendialog import OpenDialog
+        from PyQt6.QtWidgets import QFileDialog
 
         self.setStatus("Please select a file...")
         open_dialog = OpenDialog(self)
         open_dialog.setWindowTitle("Select a File")
 
-        # Use exec_() to show the dialog and get the result
-        if open_dialog.exec_() == OpenDialog.Accepted:
+        # Use exec() to show the dialog and get the result
+        if open_dialog.exec() == QFileDialog.DialogCode.Accepted:
             # Get the selected files
             selected_files = open_dialog.selectedFiles()
             if selected_files:
@@ -307,14 +335,17 @@ class MainWindow(QtWidgets.QMainWindow):
                     folder_list.insert(0, str(folder_path))
                     self.setFolderList(folder_list)
 
+                    # Set the combobox selection to the new folder
+                    self.folder.setCurrentText(str(folder_path))
+
     def doPopUp(self, message):
         """
-        User chose to open (connect with) a tiled server.
+        User chose to show a popup dialog with a message.
         """
-        from .popup import PopUp
+        from mdaviz.popup import PopUp
 
         popup = PopUp(self, message)
-        return popup.exec_() == QtWidgets.QDialog.Accepted
+        return popup.exec() == QtWidgets.QDialog.accepted
 
     def proceed(self):
         """Handle the logic when the user clicks 'OK'."""
@@ -373,7 +404,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def onFolderSelected(self, folder_name):
         """A folder was selected (from the open dialog or pull down menu)."""
-        if folder_name in [".", "", "Select a folder..."]:
+        if folder_name in [".", "", "Select a folder...", "Please select a folder..."]:
             pass
         elif folder_name == "Clear Recently Open...":
             settings.setKey(DIR_SETTINGS_KEY, "")
@@ -387,17 +418,74 @@ class MainWindow(QtWidgets.QMainWindow):
                 self.lazy_scanner.scan_folder_async(folder_path)
             else:
                 self.reset_mainwindow()
-                self.setStatus(f"\n{str(folder_path)!r} - Path does not exist.")
+                self.setStatus(f"{folder_name} does not exist or is not a directory")
 
     def onRefresh(self):
         """
         Refreshes the file list in the currently selected folder
         - Re-fetch the list of MDA files in the current folder.
         - Display the updated file list in the MDA folder table view.
+        - Invalidate cache for files in the current folder to ensure fresh data.
+        - Preserve the currently selected file after refresh.
         """
         self.setStatus("Refreshing folder...")
+        logger.info("🔄 REFRESH DEBUG: Refreshing folder...")
         current_folder = self.dataPath()
         if current_folder:
+            # Preserve the currently selected file
+            selected_file_name = None
+            if (
+                self.mvc_folder
+                and hasattr(self.mvc_folder, "mda_folder_tableview")
+                and self.mvc_folder.selectionModel()
+            ):
+                current_index = self.mvc_folder.selectionModel().currentIndex()
+                if current_index.isValid():
+                    current_file_list = self.mdaFileList()
+                    if current_index.row() < len(current_file_list):
+                        selected_file_name = current_file_list[current_index.row()]
+                        self._selected_file_name = selected_file_name
+                        self.setStatus(f"Preserving selection: {selected_file_name}")
+                    else:
+                        self.setStatus("Invalid row index for current selection")
+                else:
+                    self.setStatus("No valid selection to preserve")
+            else:
+                self.setStatus("No selection model available to preserve selection")
+
+            # Alternative approach: try to get selection from the table view directly
+            if not selected_file_name and (
+                self.mvc_folder and hasattr(self.mvc_folder, "mda_folder_tableview")
+            ):
+                table_view = self.mvc_folder.mda_folder_tableview.tableView
+                if table_view.selectionModel():
+                    current_index = table_view.selectionModel().currentIndex()
+                    if current_index.isValid():
+                        current_file_list = self.mdaFileList()
+                        if current_index.row() < len(current_file_list):
+                            selected_file_name = current_file_list[current_index.row()]
+                            self._selected_file_name = selected_file_name
+                            self.setStatus(
+                                f"Preserving selection (alt method): {selected_file_name}"
+                            )
+
+            # Invalidate cache for all files in the current folder
+            from mdaviz.data_cache import get_global_cache
+
+            cache = get_global_cache()
+            logger.info(
+                f"🔄 REFRESH DEBUG: Invalidating cache for folder: {current_folder}"
+            )
+            self.setStatus(f"Invalidating cache for folder: {current_folder}")
+            invalidated_count = cache.invalidate_folder(str(current_folder))
+            logger.info(
+                f"🔄 REFRESH DEBUG: Invalidated {invalidated_count} files from cache"
+            )
+            if invalidated_count > 0:
+                self.setStatus(f"Invalidated cache for {invalidated_count} files")
+            else:
+                self.setStatus("No cached files found to invalidate")
+
             current_mdaFileList = self.mdaFileList()
             self.onFolderSelected(current_folder)
             new_mdaFileList = self.mdaFileList()
@@ -469,13 +557,20 @@ class MainWindow(QtWidgets.QMainWindow):
         """
         if folder_list is None:
             folder_list = []
-        self.folder.clear()
-        if not folder_list:
-            folder_list = ["Select a folder..."]
-        self.folder.addItems(folder_list)
-        self.folder.addItems(["Clear Recently Open..."])
-        count = self.folder.count()
-        self.folder.insertSeparator(count - 1)
+        self.folder.clear()  # type: ignore[attr-defined]
+
+        # If no folders and auto-load is disabled, show "Please select a folder..."
+        if not folder_list and not self.get_auto_load_setting():
+            folder_list = ["Please select a folder..."]
+        else:
+            # If we have folders and auto-load is disabled, add "Please select a folder..." at the beginning
+            if not self.get_auto_load_setting():
+                folder_list = ["Please select a folder..."] + folder_list
+
+        self.folder.addItems(folder_list)  # type: ignore[attr-defined]
+        self.folder.addItems(["Clear Recently Open..."])  # type: ignore[attr-defined]
+        count = self.folder.count()  # type: ignore[attr-defined]
+        self.folder.insertSeparator(count - 1)  # type: ignore[attr-defined]
 
     def _on_scan_progress(self, current: int, total: int) -> None:
         """Handle scan progress updates."""
@@ -501,10 +596,10 @@ class MainWindow(QtWidgets.QMainWindow):
                 self.setMdaInfoList(sorted_info)
                 self.setMdaFileList(sorted_files)
                 self._addToRecentFolders(str(folder_path))
-                self.info.setText(f"{len(sorted_files)} mda files")
+                self.info.setText(f"{len(sorted_files)} mda files")  # type: ignore[attr-defined]
 
                 # Create or update the folder view
-                layout = self.groupbox.layout()
+                layout = self.groupbox.layout()  # type: ignore[attr-defined]
                 if self.mvc_folder is None:
                     self.mvc_folder = MDA_MVC(self)
                     layout.addWidget(self.mvc_folder)
@@ -512,43 +607,64 @@ class MainWindow(QtWidgets.QMainWindow):
                     # Always update the folder view since it is a new folder
                     self.mvc_folder.updateFolderView()
 
-                # Highlight the selected file if one was specified
+                # Highlight the selected file if one was specified, otherwise select the first file
                 if hasattr(self, "_selected_file_name") and self._selected_file_name:
-                    try:
-                        # Find the index of the selected file
-                        selected_index = sorted_files.index(self._selected_file_name)
-                        # Select and highlight the file in the folder view
-                        if self.mvc_folder and hasattr(
-                            self.mvc_folder, "mda_folder_tableview"
-                        ):
-                            model = (
-                                self.mvc_folder.mda_folder_tableview.tableView.model()
+                    # Use a timer to delay selection restoration (helps with timing issues on Linux)
+                    from PyQt6.QtCore import QTimer
+
+                    def restore_selection():
+                        try:
+                            # Find the index of the selected file
+                            selected_index = sorted_files.index(
+                                self._selected_file_name
+                            )
+                            # Select and highlight the file in the folder view
+                            if self.mvc_folder and hasattr(
+                                self.mvc_folder, "mda_folder_tableview"
+                            ):
+                                model = self.mvc_folder.mda_folder_tableview.tableView.model()
+                                if model and selected_index < model.rowCount():
+                                    index = model.index(selected_index, 0)
+                                    self.mvc_folder.selectAndShowIndex(index)
+                                    self.setStatus(
+                                        f"Highlighted selected file: {self._selected_file_name}"
+                                    )
+                        except ValueError:
+                            # File not found in the list, ignore
+                            self.setStatus(
+                                f"Selected file {self._selected_file_name} not found after refresh"
+                            )
+                        finally:
+                            # Clear the selected file name
+                            self._selected_file_name = None
+
+                    # Use a short delay to ensure the model is fully updated
+                    QTimer.singleShot(100, restore_selection)
+                else:
+                    # Auto-select the first file if no specific file was selected
+                    if self.mvc_folder and hasattr(
+                        self.mvc_folder, "mda_folder_tableview"
+                    ):
+                        model = self.mvc_folder.mda_folder_tableview.tableView.model()
+                        if model and model.rowCount() > 0:
+                            first_index = model.index(0, 0)
+                            self.mvc_folder.selectAndShowIndex(first_index)
+                            self.setStatus(
+                                f"Auto-selected first file: {sorted_files[0]}"
                             )
-                            if model and selected_index < model.rowCount():
-                                index = model.index(selected_index, 0)
-                                self.mvc_folder.selectAndShowIndex(index)
-                                self.setStatus(
-                                    f"Highlighted selected file: {self._selected_file_name}"
-                                )
-                    except ValueError:
-                        # File not found in the list, ignore
-                        pass
-                    finally:
-                        # Clear the selected file name
-                        self._selected_file_name = None
 
                 self.setStatus(
                     f"Loaded {len(sorted_files)} MDA files from {folder_path}"
                 )
             else:
                 error_msg = "Could not determine folder path from scan results"
-                self.info.setText("No mda files")
+                self.info.setText("No mda files")  # type: ignore[attr-defined]
                 self.doPopUp(error_msg)
                 self.reset_mainwindow()
                 self.setStatus(f"Scan failed: {error_msg}")
         else:
             error_msg = result.error_message or "No MDA files found"
-            self.info.setText("No mda files")
+            self.info.setText("No mda files")  # type: ignore[attr-defined]
             self.doPopUp(error_msg)
             self.reset_mainwindow()
             self.setStatus(f"Scan failed: {error_msg}")
@@ -564,31 +680,36 @@ class MainWindow(QtWidgets.QMainWindow):
         # Ensure central widget is resizable
         if hasattr(self, "centralwidget"):
             self.centralwidget.setSizePolicy(
-                QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
+                QtWidgets.QSizePolicy.Policy.Expanding,
+                QtWidgets.QSizePolicy.Policy.Expanding,
             )
 
         # Ensure the main content area (groupbox) is resizable
         if hasattr(self, "groupbox"):
             self.groupbox.setSizePolicy(
-                QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
+                QtWidgets.QSizePolicy.Policy.Expanding,
+                QtWidgets.QSizePolicy.Policy.Expanding,
             )
 
         # Ensure the tab widget is resizable
         if hasattr(self, "mainTabWidget"):
             self.mainTabWidget.setSizePolicy(
-                QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
+                QtWidgets.QSizePolicy.Policy.Expanding,
+                QtWidgets.QSizePolicy.Policy.Expanding,
             )
 
         # Ensure the fit data tab is resizable
         if hasattr(self, "fitDataTab"):
             self.fitDataTab.setSizePolicy(
-                QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
+                QtWidgets.QSizePolicy.Policy.Expanding,
+                QtWidgets.QSizePolicy.Policy.Expanding,
             )
 
         # Ensure the fit data text widget is resizable
         if hasattr(self, "fitDataText"):
             self.fitDataText.setSizePolicy(
-                QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
+                QtWidgets.QSizePolicy.Policy.Expanding,
+                QtWidgets.QSizePolicy.Policy.Expanding,
             )
 
     def _auto_load_first_folder(self) -> None:
@@ -601,12 +722,8 @@ class MainWindow(QtWidgets.QMainWindow):
 
         The auto-loading can be disabled by setting the 'auto_load_folder' setting to False.
         """
-        # Check if auto-loading is enabled (default to True)
-        auto_load_enabled = settings.getKey("auto_load_folder")
-        if auto_load_enabled is None:
-            # Default to True if not set
-            auto_load_enabled = True
-            settings.setKey("auto_load_folder", True)
+        # Check if auto-loading is enabled using the proper boolean conversion
+        auto_load_enabled = self.get_auto_load_setting()
 
         if not auto_load_enabled:
             self.setStatus("Auto-loading disabled by user preference")
@@ -668,7 +785,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def _center_window(self):
         """Center the window on the screen."""
-        screen = QtWidgets.QDesktopWidget().screenGeometry()
+        screen = QtWidgets.QApplication.primaryScreen().geometry()
         window_geometry = self.geometry()
         x = (screen.width() - window_geometry.width()) // 2
         y = (screen.height() - window_geometry.height()) // 2
