@@ -7,39 +7,47 @@ Display content of the currently selected files.
     ~tabManager
 
 User: tabCloseRequested.connect (emit: index)
-        --> onTabCloseRequested(index --> file_path)
-        --> tabManager.removeTab(file_path)
-        --> tabManager.tabRemoved.emit(file_path)
-        --> onTabRemoved(file_path --> index)
+
+    --> onTabCloseRequested(index --> file_path)
+    --> tabManager.removeTab(file_path)
+    --> tabManager.tabRemoved.emit(file_path)
+    --> onTabRemoved(file_path --> index)
 
 User: clearButton.clicked (emit: no data)
-        --> onClearAllTabsRequested()
-        --> tabManager.removeAllTabs()
-        --> tabManager.allTabsRemoved.emit()
-        --> onAllTabsRemoved()
-        --> removeAllFileTabs()
+
+    --> onClearAllTabsRequested()
+    --> tabManager.removeAllTabs()
+    --> tabManager.allTabsRemoved.emit()
+    --> onAllTabsRemoved()
+    --> removeAllFileTabs()
 
 
 
 """
 
-from .synApps_mdalib.mda import readMDA
-from PyQt5 import QtCore
-from PyQt5 import QtWidgets
-from PyQt5.QtWidgets import QAbstractItemView
+from mdaviz.synApps_mdalib.mda import readMDA
+from PyQt6.QtCore import pyqtSignal
+from PyQt6.QtWidgets import QAbstractItemView, QWidget
+from PyQt6.QtCore import QObject
 import yaml
 
-from . import utils
-from .mda_file_table_view import MDAFileTableView
-from .data_cache import get_global_cache
+from mdaviz import utils
+from mdaviz.mda_file_table_view import MDAFileTableView
+from mdaviz.data_cache import get_global_cache
+from mdaviz.logger import get_logger
+
+# Get logger for this module
+logger = get_logger("mda_file")
 
 
-class MDAFile(QtWidgets.QWidget):
+class MDAFile(QWidget):
     ui_file = utils.getUiFileName(__file__)
     # Emit the action when button is pushed:
-    buttonPushed = QtCore.pyqtSignal(str)
+    buttonPushed = pyqtSignal(str)
     # Emit the new tab index (int), file path (str), data (dict) and selection field (dict):
-    tabChanged = QtCore.pyqtSignal(int, str, dict, dict)
+    tabChanged = pyqtSignal(int, str, dict, dict)
+    # Emit X2 value changes for 2D data:
+    x2ValueChanged = pyqtSignal(int)
 
     def __init__(self, parent):
         """
@@ -168,10 +176,10 @@ class MDAFile(QtWidgets.QWidget):
         file_path = self.dataPath() / file_name
 
         # Debug: print the paths to see what's happening
-        print(f"Debug - folder_path: {folder_path}")
-        print(f"Debug - file_name: {file_name}")
-        print(f"Debug - file_path: {file_path}")
-        print(f"Debug - file_path.exists(): {file_path.exists()}")
+        logger.debug(f"folder_path: {folder_path}")
+        logger.debug(f"file_name: {file_name}")
+        logger.debug(f"file_path: {file_path}")
+        logger.debug(f"file_path.exists(): {file_path.exists()}")
 
         # Use data cache for better performance
         cache = get_global_cache()
@@ -189,6 +197,13 @@ class MDAFile(QtWidgets.QWidget):
                 "firstDet": cached_data.first_det,
                 "pvList": cached_data.pv_list,
                 "index": index,
+                # Add 2D data support
+                "scanDict2D": cached_data.scan_dict_2d,
+                "scanDictInner": cached_data.scan_dict_inner,
+                "isMultidimensional": cached_data.is_multidimensional,
+                "rank": cached_data.rank,
+                "dimensions": cached_data.dimensions,
+                "acquiredDimensions": cached_data.acquired_dimensions,
             }
         else:
             # Fallback to direct loading if cache fails
@@ -206,16 +221,48 @@ class MDAFile(QtWidgets.QWidget):
                     "firstDet": 1,
                     "pvList": [],
                     "index": index,
+                    # Add 2D data support
+                    "scanDict2D": {},
+                    "scanDictInner": {},
+                    "isMultidimensional": False,
+                    "rank": 1,
+                    "dimensions": [],
+                    "acquiredDimensions": [],
                 }
                 return
 
             file_metadata, file_data_dim1, *_ = result
-            if file_metadata["rank"] > 1:
-                self.setStatus(
-                    "WARNING: Multidimensional data not supported - ignoring ranks > 1."
-                )
+            rank = file_metadata.get("rank", 1)
+            dimensions = file_metadata.get("dimensions", [])
+            acquired_dimensions = file_metadata.get("acquired_dimensions", [])
+
             scanDict, first_pos, first_det = utils.get_scan(file_data_dim1)
-            pvList = [v["name"] for v in scanDict.values()]
+
+            # Initialize 2D data fields
+            scanDict2D = {}
+            scan_dict_inner = {}
+            is_multidimensional = rank > 1
+
+            # Process 2D data if available
+            if rank >= 2 and len(result) > 2:
+                try:
+                    file_data_dim2 = result[2]
+                    scanDict2D, _, _ = utils.get_scan_2d(file_data_dim1, file_data_dim2)
+                    # Also store inner dimension data for 1D plotting
+                    scan_dict_inner, _, _ = utils.get_scan(file_data_dim2)
+                except Exception as e:
+                    logger.warning(f"Warning: Could not process 2D data: {e}")
+                    scanDict2D = {}
+                    scan_dict_inner = {}
+
+            # Construct pvList from appropriate data source
+            # For 2D+ data, use scan_dict_inner (inner dimension PVs like P1, D1, etc.)
+            # For 1D data, use scanDict (outer dimension PVs)
+            if is_multidimensional and scan_dict_inner:
+                pvList = [v["name"] for v in scan_dict_inner.values()]
+            else:
+                pvList = [v["name"] for v in scanDict.values()]
+
             self._data = {
                 "fileName": file_path.stem,  # file_name.rsplit(".mda", 1)[0]
                 "filePath": str(file_path),
@@ -226,8 +273,61 @@ class MDAFile(QtWidgets.QWidget):
                 "firstDet": first_det,
                 "pvList": pvList,
                 "index": index,
+                # Add 2D data support
+                "scanDict2D": scanDict2D,
+                "scanDictInner": scan_dict_inner,
+                "isMultidimensional": is_multidimensional,
+                "rank": rank,
+                "dimensions": dimensions,
+                "acquiredDimensions": acquired_dimensions,
             }
 
+    def handle2DMode(self):
+        """Handle 2D data setup - update controls but don't change mode."""
+        if self._data.get("isMultidimensional", False):
+            # Update 2D controls in table view
+            table_view = self.tabIndex2Tableview(self.tabWidget.currentIndex())
+            if table_view:
+                dimensions = self._data.get("dimensions", [])
+                acquired_dimensions = self._data.get("acquiredDimensions", [])
+                logger.debug(f"handle2DMode - dimensions: {dimensions}")
+                logger.debug(
+                    f"handle2DMode - acquired_dimensions: {acquired_dimensions}"
+                )
+
+                # Extract X2 positioner information from scanDict2D
+                x2_positioner_info = None
+                scan_dict_2d = self._data.get("scanDict2D", {})
+                logger.debug(
+                    f"handle2DMode - scanDict2D keys: {list(scan_dict_2d.keys())}"
+                )
+                if scan_dict_2d:
+                    # Find the X2 positioner (first positioner in 2D data)
+                    # In 2D data, the first positioner (index 0) is typically X2
+                    for key, value in scan_dict_2d.items():
+                        logger.debug(
+                            f"handle2DMode - Field {key}: type={value.get('type')}, name={value.get('name')}"
+                        )
+                        if (
+                            value.get("type") == "POS" and key == 0
+                        ):  # First positioner is X2
+                            x2_positioner_info = {
+                                "name": value.get("name", "X2"),
+                                "unit": value.get("unit", ""),
+                                "data": value.get("data", []),
+                            }
+                            logger.debug(
+                                f"handle2DMode - X2 positioner found: {x2_positioner_info['name']}"
+                            )
+                            break
+
+                table_view.update2DControls(
+                    is_multidimensional=self._data.get("isMultidimensional", False),
+                    dimensions=dimensions,
+                    acquired_dimensions=acquired_dimensions,
+                    x2_positioner_info=x2_positioner_info,
+                )
+
     def setStatus(self, text):
         self.mda_mvc.setStatus(text)
 
@@ -279,7 +379,8 @@ class MDAFile(QtWidgets.QWidget):
         """Display pos(s) & det(s) values as a tableview in the vizualization panel."""
         if not self.data():
             return
-        self.mda_mvc.mda_file_viz.setTableData(tabledata)
+        # Pass full data structure instead of just scanDict for 2D support
+        self.mda_mvc.mda_file_viz.setTableData(self.data())
 
     def defaultSelection(self, first_pos, first_det, selection_field):
         """
@@ -371,11 +472,24 @@ class MDAFile(QtWidgets.QWidget):
         self.displayData(tabledata)
         selection_field = self.defaultSelection(first_pos, first_det, selection_field)
         # Update tab widget:
+        mode = self.mode()
         if self.tabManager.getTabData(file_path):
-            tab_index = self.tabPath2Index(file_path)
-            self.tabWidget.setCurrentIndex(tab_index)
+            # File already exists in tab manager
+            if mode in ("Auto-replace"):
+                # In auto-replace mode, clear all tabs and recreate this one
+                while self.tabWidget.count() > 0:
+                    self.tabWidget.removeTab(0)
+                self.tabManager.removeAllTabs()
+                # Add this tab to the UI:
+                self.createNewTab(file_name, file_path, selection_field)
+                # Add new tab to tabManager:
+                self.tabManager.addTab(file_path, metadata, tabledata)
+            else:
+                # In auto-add/auto-off mode, just switch to existing tab
+                tab_index = self.tabPath2Index(file_path)
+                self.tabWidget.setCurrentIndex(tab_index)
         else:
-            mode = self.mode()
+            # File is new
             if mode in ("Auto-add", "Auto-off"):
                 # Add this tab to the UI:
                 self.createNewTab(file_name, file_path, selection_field)
@@ -389,6 +503,21 @@ class MDAFile(QtWidgets.QWidget):
             # Add new tab to tabManager:
             self.tabManager.addTab(file_path, metadata, tabledata)
 
+        # Handle mode switching for 2D data AFTER tab management is complete
+        if self._data.get("isMultidimensional", False):
+            self.handle2DMode()
+            # Set 2D data in visualization
+            self.mda_mvc.mda_file_viz.set2DData(self._data)
+        else:
+            # Clear 2D data for 1D files
+            self.mda_mvc.mda_file_viz.set2DData(None)
+            # Hide 2D controls for 1D files
+            table_view = self.tabIndex2Tableview(self.tabWidget.currentIndex())
+            if table_view:
+                table_view.update2DControls(
+                    is_multidimensional=False, dimensions=None, x2_positioner_info=None
+                )
+
     def createNewTab(self, file_name, file_path, selection_field):
         """
         Creates and activates a new tab with a MDAFileTableView for the selected file.
@@ -481,11 +610,11 @@ class MDAFile(QtWidgets.QWidget):
         model = tableview.tableView.model()
         model.setHighlightRow(row)
         rowCount = model.rowCount()
-        scrollHint = QAbstractItemView.EnsureVisible
+        scrollHint = QAbstractItemView.ScrollHint.EnsureVisible
         if row == 0:
-            scrollHint = QAbstractItemView.PositionAtTop
+            scrollHint = QAbstractItemView.ScrollHint.PositionAtTop
         elif row == rowCount - 1:
-            scrollHint = QAbstractItemView.PositionAtBottom
+            scrollHint = QAbstractItemView.ScrollHint.PositionAtBottom
         # Get the QModelIndex for the specified row
         index = model.index(row, 0)
         tableview.tableView.scrollTo(index, scrollHint)
@@ -497,7 +626,7 @@ class MDAFile(QtWidgets.QWidget):
         action:
             from buttons: add, clear, replace
         """
-        print(f"\nResponder: {action=}")
+        logger.debug(f"\nResponder: {action=}")
         self.buttonPushed.emit(action)
 
     def updateButtonVisibility(self):
@@ -513,7 +642,7 @@ class MDAFile(QtWidgets.QWidget):
 # ------ Tabs management (data):
 
 
-class TabManager(QtCore.QObject):
+class TabManager(QObject):
     """
     Manages the content of the currently opened tabs (data aspect only).
 
@@ -531,9 +660,9 @@ class TabManager(QtCore.QObject):
     - allTabRemoved: Emitted when all tabs are removed. No parameters.
     """
 
-    tabAdded = QtCore.pyqtSignal(str)  # Signal emitting file path of removed tab
-    tabRemoved = QtCore.pyqtSignal(str)  # Signal emitting file path of removed tab
-    allTabsRemoved = QtCore.pyqtSignal()  # Signal indicating all tabs have been removed
+    tabAdded = pyqtSignal(str)  # Signal emitting file path of removed tab
+    tabRemoved = pyqtSignal(str)  # Signal emitting file path of removed tab
+    allTabsRemoved = pyqtSignal()  # Signal indicating all tabs have been removed
 
     def __init__(self):
         super().__init__()
