@@ -66,19 +66,22 @@ import time
 from functools import partial
 from pathlib import Path
 
-from PyQt5 import QtWidgets
-from PyQt5 import QtCore
-from PyQt5.QtCore import QItemSelectionModel
-from PyQt5.QtWidgets import QAbstractItemView
+from PyQt6 import QtCore
+from PyQt6.QtCore import QItemSelectionModel, Qt, pyqtSignal
+from PyQt6.QtWidgets import QAbstractItemView, QWidget
 
-from . import utils
+from mdaviz import utils
+from mdaviz.logger import get_logger
+
+# Get logger for this module
+logger = get_logger("mda_folder")
 
 
-class MDA_MVC(QtWidgets.QWidget):
+class MDA_MVC(QWidget):
     """Model View Controller class for mda files."""
 
     # DESIGN NOTE: Should this signal be emitted here or in MDA_FILE_TM? Review if refactoring signal emission logic.
-    detRemoved = QtCore.pyqtSignal(
+    detRemoved = pyqtSignal(
         str, int
     )  # Emit the file path and row when a DET checkbox is unchecked
 
@@ -101,10 +104,10 @@ class MDA_MVC(QtWidgets.QWidget):
         self.setup()
 
     def setup(self):
-        from .user_settings import settings
-        from .mda_folder_table_view import MDAFolderTableView
-        from .mda_file_viz import MDAFileVisualization
-        from .mda_file import MDAFile
+        from mdaviz.user_settings import settings
+        from mdaviz.mda_folder_table_view import MDAFolderTableView
+        from mdaviz.mda_file_viz import MDAFileVisualization
+        from mdaviz.mda_file import MDAFile
 
         # Folders table view:
         self.mda_folder_tableview = MDAFolderTableView(self)
@@ -134,12 +137,12 @@ class MDA_MVC(QtWidgets.QWidget):
             selection_model = self.mda_folder_tableview.tableView.selectionModel()
             self.setSelectionModel(selection_model)
         # Ensure focus policy and selection mode for keyboard navigation
-        self.mda_folder_tableview.tableView.setFocusPolicy(QtCore.Qt.StrongFocus)
+        self.mda_folder_tableview.tableView.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
         self.mda_folder_tableview.tableView.setSelectionMode(
-            QAbstractItemView.SingleSelection
+            QAbstractItemView.SelectionMode.SingleSelection
         )
         self.mda_folder_tableview.tableView.setSelectionBehavior(
-            QAbstractItemView.SelectRows
+            QAbstractItemView.SelectionBehavior.SelectRows
         )
 
         # Folder table view signal/slot connections:
@@ -158,6 +161,9 @@ class MDA_MVC(QtWidgets.QWidget):
         # MDAFile Push buttons connection:
         utils.reconnect(self.mda_file.buttonPushed, self.doPlot)
 
+        # MDAFile X2 value changes connection:
+        utils.reconnect(self.mda_file.x2ValueChanged, self.doPlot)
+
         # Debug signals:
         # self.mda_folder_tableview.tableView.doubleClicked.connect(utils.debug_signal)
         # self.mda_folder_tableview.firstButton.clicked.connect(utils.debug_signal)
@@ -305,11 +311,11 @@ class MDA_MVC(QtWidgets.QWidget):
             None: Updates the selection field directly if changes are made.
         """
         if not old_selection:
-            print("No previous selection.")
+            logger.debug("No previous selection.")
             return
 
         if verbose:
-            print(f"\n----- Selection before clean up: {old_selection}")
+            logger.debug(f"\n----- Selection before clean up: {old_selection}")
         changes_made = False
         tableview = self.currentFileTableview()
         new_selection = {"Y": [], "X": 0}
@@ -338,26 +344,30 @@ class MDA_MVC(QtWidgets.QWidget):
                     if new_i0_idx != old_i0_idx:
                         changes_made = True
                         if verbose:
-                            print(
+                            logger.debug(
                                 f"I0 <{old_i0_pv}> changed from {old_i0_idx} to {new_i0_idx}"
                             )
                 else:
                     changes_made = True
                     if verbose:
-                        print(f"I0 <{old_i0_pv}> was removed - auto-unchecking I0")
+                        logger.debug(
+                            f"I0 <{old_i0_pv}> was removed - auto-unchecking I0"
+                        )
                     # I0 PV doesn't exist in new file, so uncheck the I0 checkbox
                     tableview.tableView.model().uncheckCheckBox(old_i0_idx)
             else:
                 changes_made = True
                 if verbose:
-                    print(f"I0 index {old_i0_idx} out of range - auto-unchecking I0")
+                    logger.debug(
+                        f"I0 index {old_i0_idx} out of range - auto-unchecking I0"
+                    )
                 # I0 index was invalid, so uncheck the I0 checkbox
                 tableview.tableView.model().uncheckCheckBox(old_i0_idx)
 
         if changes_made:
             self.applySelectionChanges(new_selection)
         if verbose:
-            print(f"----- Selection After clean up: {self.selectionField()}\n")
+            logger.debug(f"----- Selection After clean up: {self.selectionField()}\n")
 
     def updateDetectorSelection(
         self, oldPvList, old_selection, newPvList, new_selection, verbose=False
@@ -378,13 +388,13 @@ class MDA_MVC(QtWidgets.QWidget):
                     if new_index != old_index:
                         changes_made = True
                         if verbose:
-                            print(
+                            logger.debug(
                                 f"DET <{old_pv}> changed from {old_index} to {new_index}"
                             )
                 else:
                     changes_made = True
                     if verbose:
-                        print(f"DET <{old_pv}> was removed")
+                        logger.debug(f"DET <{old_pv}> was removed")
         return changes_made
 
     # # ------------ File selection methods:
@@ -417,7 +427,8 @@ class MDA_MVC(QtWidgets.QWidget):
         """
 
         selected_file = self.mdaFileList()[index.row()]
-        self.setStatus(f"\nLoading {str(self.dataPath())}/{selected_file}")
+        file_path = f"{str(self.dataPath())}/{selected_file}"
+        self.setStatus(f"\nLoading {file_path}")
 
         # Ensures the table view scrolls to the selected item.
         if isinstance(index, QtCore.QModelIndex):
@@ -453,7 +464,7 @@ class MDA_MVC(QtWidgets.QWidget):
                         old_selection, old_pv_list, new_pv_list, verbose=False
                     )
                 except Exception as exc:
-                    print(str(exc))
+                    logger.error(str(exc))
 
             self.handlePlotBasedOnMode()
         else:
@@ -479,34 +490,75 @@ class MDA_MVC(QtWidgets.QWidget):
         - Uses a ChartView widget for plotting within a specified layout.
 
         Parameters:
-        - args[0] (str): The plotting action to be taken ('add', 'replace', or 'clear').
+        - args[0] (str or int): The plotting action to be taken ('add', 'replace', or 'clear'),
+          or an integer representing the X2 value change.
         - kwargs (dict): Additional options for plotting, if any.
 
         Behavior:
         - 'clear': Removes all tabs and clears the visualization.
         - 'add'/'replace': Plots new data according to the current selection, adding to existing plots or replacing them.
+        - X2 value change: Replots the current selection with the new X2 slice.
         - Exits with a status message if no file is selected or no detectors (Y) are selected for plotting.
         """
 
-        from .chartview import ChartView
+        from mdaviz.chartview import ChartView
+
+        # Handle X2 value changes
+        if isinstance(args[0], int):
+            # This is an X2 value change, respect the current mode
+            mode = self.mda_file.mode()
+            if mode == "Auto-add":
+                action = "add"
+            elif mode == "Auto-off":
+                # Do nothing in auto-off mode - user must manually add/replace
+                logger.debug(
+                    f"\ndoPlot called for X2 value change: X2={args[0]}, mode={mode}, action=ignore"
+                )
+                return
+            else:
+                action = "replace"  # Default for Auto-replace
+            logger.debug(
+                f"\ndoPlot called for X2 value change: X2={args[0]}, mode={mode}, action={action}"
+            )
+        else:
+            action = args[0]
+            logger.debug(f"\ndoPlot called: action={action}")
 
-        action = args[0]
         tableview = self.currentFileTableview()
+
+        # Check if this is a 2D selection (has X1, X2 keys)
+        is_2d_selection = (
+            len(args) > 1
+            and isinstance(args[1], dict)
+            and "X1" in args[1]
+            and "X2" in args[1]
+        )
+        logger.debug(f"doPlot - is_2d_selection: {is_2d_selection}")
+
+        if is_2d_selection:
+            # Handle 2D plotting - use the passed selection
+            selection = args[1]
+            logger.debug("doPlot - Handling 2D plotting")
+            self._doPlot2D(action, selection)
+            return
+
+        # For 1D plotting, use the selection from the table view
         selection = self.selectionField()
 
         # Clear all content from the file table view & viz panel:
         if action in ("clear"):
-            print("\n\nPushButton Clear...")
+            logger.debug("\n\nPushButton Clear...")
             self.mda_file.removeAllFileTabs()
             return
 
-        # If there is no file selected (ie no tableview) or no DET:
-        if not tableview or not selection.get("Y"):
+        # If there is no file selected (ie no tableview) or no selection or no DET:
+        if not tableview or not selection or not selection.get("Y"):
             self.setStatus("Nothing to plot.")
             return
 
-        print(f"\ndoPlot called: {action=}, {selection=}")
+        logger.debug(f"\ndoPlot called: {action=}, {selection=}")
 
+        # Handle 1D plotting (existing logic)
         # Get the matplotlib chartview widget, if exists:
         layoutMpl = self.mda_file_viz.plotPageMpl.layout()
         if layoutMpl.count() != 1:  # in case something changes ...
@@ -516,12 +568,19 @@ class MDA_MVC(QtWidgets.QWidget):
         if action in ("replace", "add"):
             # Get dataset for the positioner/detector selection:
             datasets, plot_options = tableview.data2Plot(selection)
+            logger.debug(f"doPlot - Received plot_options: {plot_options}")
             y_index = selection.get("Y", [])
             if not isinstance(widgetMpl, ChartView):  # Make a blank chart.
                 widgetMpl = ChartView(self, **plot_options)
                 # Connect fit signals to main window
                 if hasattr(self.mainWindow, "connectToFitSignals"):
                     self.mainWindow.connectToFitSignals(widgetMpl)
+
+                # Apply stored log scale state to the new chart
+                if hasattr(self.mda_file_viz, "getLogScaleState"):
+                    stored_log_x, stored_log_y = self.mda_file_viz.getLogScaleState()
+                    widgetMpl.setLogScales(stored_log_x, stored_log_y)
+
             if action in ("replace"):
                 widgetMpl.curveManager.removeAllCurves()
             for i, (ds, ds_options) in zip(y_index, datasets):
@@ -529,9 +588,66 @@ class MDA_MVC(QtWidgets.QWidget):
                 # ds_options: {"label":y_label} (for legend)
                 # plot_options: {"x" (label), "x_unit", "y" (label), "y_unit", "title", "folderPath"}
                 options = {"ds_options": ds_options, "plot_options": plot_options}
+                # Add X2 index to options if available
+                if "x2_index" in plot_options:
+                    options["x2_index"] = plot_options["x2_index"]
+                    logger.debug(
+                        f"doPlot - Passing x2_index: {plot_options['x2_index']}"
+                    )
+                else:
+                    logger.debug("doPlot - No x2_index in plot_options")
                 widgetMpl.plot(i, *ds, **options)
             self.mda_file_viz.setPlot(widgetMpl)
 
+    def _doPlot2D(self, action, selection):
+        """
+        Handle 2D plotting with the given selection.
+
+        Parameters:
+            action (str): 'add' or 'replace'
+            selection (dict): 2D selection with X1, X2, Y, I0, plot_type keys
+        """
+        logger.debug(f"_doPlot2D - action: {action}, selection: {selection}")
+
+        tableview = self.currentFileTableview()
+        if not tableview:
+            logger.debug("_doPlot2D - No tableview available")
+            return
+
+        # Convert 2D selection to 1D format for data2Plot2D
+        # data2Plot2D expects: {'X': x_index, 'Y': [y_indices], 'I0': i0_index}
+        converted_selection = {
+            "X": selection.get("X1"),
+            "Y": selection.get("Y", []),
+            "I0": selection.get("I0"),
+            "log_y": selection.get("log_y", False),
+        }
+
+        logger.debug(f"_doPlot2D - Converted selection: {converted_selection}")
+
+        # Get 2D data and plot options
+        try:
+            datasets, plot_options = tableview.data2Plot2D(converted_selection)
+            logger.debug(
+                f"_doPlot2D - Got datasets: {len(datasets)}, plot_options: {plot_options}"
+            )
+
+            # Update the 2D plot in the 2D tab
+            self.mda_file_viz.update2DPlot()
+
+            # Set the plot type in the 2D chart view
+            plot_type = selection.get("plot_type", "heatmap")
+            if (
+                hasattr(self.mda_file_viz, "widgetMpl2D")
+                and self.mda_file_viz.widgetMpl2D
+            ):
+                self.mda_file_viz.widgetMpl2D.set_plot_type(plot_type)
+                logger.debug(f"_doPlot2D - Set plot type to: {plot_type}")
+
+        except Exception as e:
+            logger.debug(f"_doPlot2D - Error: {e}")
+            self.setStatus(f"Error plotting 2D data: {e}")
+
     def onTabChanged(self, index, file_path, file_data, selection_field):
         """
         Updates UI to reflect the content of the newly selected tab or resets UI if no tab is selected.
@@ -568,9 +684,65 @@ class MDA_MVC(QtWidgets.QWidget):
 
         # Fetch and display the metadata and data associated with the file path
         if file_data:
-            print(f"Displaying data and metadata for {Path(file_path).name}.")
+            logger.debug(f"Displaying data and metadata for {Path(file_path).name}.")
             self.mda_file.displayMetadata(file_data.get("metadata", None))
-            self.mda_file.displayData(file_data.get("tabledata", None))
+            # Reconstruct full data structure for 2D support by setting data again
+            # This ensures we have scanDict, scanDict2D, isMultidimensional, etc.
+            if file_path:
+                # Find the file index in the current folder
+                try:
+                    file_name = Path(file_path).name
+                    file_index = self.mdaFileList().index(file_name)
+                    # Set the data again to get the full structure
+                    self.mda_file.setData(file_index)
+                    # Now display the full data structure
+                    self.mda_file.displayData(self.mda_file.data())
+                except ValueError:
+                    # File not found in current folder, use fallback
+                    self.mda_file.displayData(file_data)
+
+        # Highlight the corresponding file in the folder table view if it belongs to the current folder
+        if file_path:
+            file_name = Path(file_path).name
+            current_folder_path = str(self.dataPath())
+
+            # Check if the file belongs to the currently loaded folder
+            if file_path.startswith(current_folder_path):
+                try:
+                    # Find the index of the file in the current folder's file list
+                    file_index = self.mdaFileList().index(file_name)
+
+                    # Get the model and create the index
+                    model = self.mda_folder_tableview.tableView.model()
+                    if model and file_index < model.rowCount():
+                        index = model.index(file_index, 0)
+
+                        # Temporarily disconnect the currentChanged signal to avoid triggering onFileSelected
+                        if self.selectionModel():
+                            self.selectionModel().currentChanged.disconnect(
+                                self.onFileSelected
+                            )
+
+                        # Highlight the file in the folder table view
+                        self.mda_folder_tableview.tableView.setFocus()
+                        self.selectionModel().setCurrentIndex(
+                            index,
+                            QItemSelectionModel.SelectionFlag.ClearAndSelect
+                            | QItemSelectionModel.SelectionFlag.Rows,
+                        )
+                        self.mda_folder_tableview.tableView.scrollTo(
+                            index, QAbstractItemView.ScrollHint.EnsureVisible
+                        )
+
+                        # Reconnect the currentChanged signal
+                        if self.selectionModel():
+                            self.selectionModel().currentChanged.connect(
+                                self.onFileSelected
+                            )
+
+                except ValueError:
+                    # File not found in the current folder's file list, ignore
+                    pass
 
         # NOTE: Consider disabling UI elements or actions that require an active file/folder to be meaningful.
 
@@ -607,7 +779,7 @@ class MDA_MVC(QtWidgets.QWidget):
                 if nextIndex.isValid():
                     self.selectAndShowIndex(nextIndex)
         except RuntimeError as e:
-            print("The selection model is no longer valid:", e)
+            logger.error("The selection model is no longer valid:", e)
 
     def goToPrevious(self):
         """
@@ -622,7 +794,7 @@ class MDA_MVC(QtWidgets.QWidget):
                 if prevIndex.isValid():
                     self.selectAndShowIndex(prevIndex)
         except RuntimeError as e:
-            print("The selection model is no longer valid:", e)
+            logger.error("The selection model is no longer valid:", e)
 
     def selectAndShowIndex(self, index):
         """
@@ -641,14 +813,16 @@ class MDA_MVC(QtWidgets.QWidget):
         # Determine the appropriate scrollHint based on the row position:
         model = self.mda_folder_tableview.tableView.model()
         rowCount = model.rowCount()
-        scrollHint = QAbstractItemView.EnsureVisible
+        scrollHint = QAbstractItemView.ScrollHint.EnsureVisible
         if index.row() == 0:
-            scrollHint = QAbstractItemView.PositionAtTop
+            scrollHint = QAbstractItemView.ScrollHint.PositionAtTop
         elif index.row() == rowCount - 1:
-            scrollHint = QAbstractItemView.PositionAtBottom
+            scrollHint = QAbstractItemView.ScrollHint.PositionAtBottom
         # Select the row and ensure it's visible:
         self.selectionModel().setCurrentIndex(
-            index, QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows
+            index,
+            QItemSelectionModel.SelectionFlag.ClearAndSelect
+            | QItemSelectionModel.SelectionFlag.Rows,
         )
         self.mda_folder_tableview.tableView.scrollTo(index, scrollHint)
         # Trigger actions associated with file selection
@@ -676,7 +850,7 @@ class MDA_MVC(QtWidgets.QWidget):
         - If I0 is selected, Y data will be normalized as Y/I0.
         """
 
-        from .chartview import ChartView
+        from mdaviz.chartview import ChartView
 
         mode = self.mda_file.mode()
         tableview = self.currentFileTableview()
@@ -695,7 +869,8 @@ class MDA_MVC(QtWidgets.QWidget):
             return
 
         # If no POS, default to index:
-        if not selection.get("X"):
+        if not selection or not selection.get("X"):
+            selection = selection or {}
             selection["X"] = 0
             tableview.tableView.model().checkCheckBox(0, "X")
 
@@ -704,28 +879,40 @@ class MDA_MVC(QtWidgets.QWidget):
             widgetMpl.curveManager.removeAllCurves()
             return
 
-        # If a DET is uncheked:
+        # Handle detector removal - emit signals but don't return early
         if det_removed:
             for y in det_removed:
                 tab_index = self.mda_file.tabWidget.currentIndex()
                 file_path = self.mda_file.tabIndex2Path(tab_index)
                 self.detRemoved.emit(file_path, y)
-                return
 
         # Get dataset for the positioner/detector selection:
         datasets, plot_options = tableview.data2Plot(selection)
+        logger.debug(f"doPlot - Received plot_options: {plot_options}")
 
         if not isinstance(widgetMpl, ChartView):
             widgetMpl = ChartView(self, **plot_options)  # Make a blank chart.
             # Connect fit signals to main window
             if hasattr(self.mainWindow, "connectToFitSignals"):
                 self.mainWindow.connectToFitSignals(widgetMpl)
+
+            # Apply stored log scale state to the new chart
+            if hasattr(self.mda_file_viz, "getLogScaleState"):
+                stored_log_x, stored_log_y = self.mda_file_viz.getLogScaleState()
+                widgetMpl.setLogScales(stored_log_x, stored_log_y)
+
         if mode in ("Auto-replace"):
             widgetMpl.curveManager.removeAllCurves()
         for row, (ds, ds_options) in zip(new_y_selection, datasets):
             # ds_options: label (for legend)
             # plot_options: xlabel, ylabel, title
             options = {"ds_options": ds_options, "plot_options": plot_options}
+            # Add X2 index to options if available
+            if "x2_index" in plot_options:
+                options["x2_index"] = plot_options["x2_index"]
+                logger.debug(f"doPlot - Passing x2_index: {plot_options['x2_index']}")
+            else:
+                logger.debug("doPlot - No x2_index in plot_options")
             widgetMpl.plot(row, *ds, **options)
         self.mda_file_viz.setPlot(widgetMpl)
 
@@ -769,7 +956,7 @@ class MDA_MVC(QtWidgets.QWidget):
         Parameters:
         - key (str): Identifier for the splitter being monitored '(hsplitter' or 'vsplitter')
         """
-        from .user_settings import settings
+        from mdaviz.user_settings import settings
 
         splitter = getattr(self, key)
         while time.time() < getattr(self, f"{key}_deadline"):
@@ -782,7 +969,65 @@ class MDA_MVC(QtWidgets.QWidget):
     # # ------------ Status method
 
     def setStatus(self, text):
-        """
-        Updates the application's status bar with the provided message.
-        """
+        """Set status text."""
         self.mainWindow.setStatus(text)
+
+    def getCurrentFilePath(self):
+        """Get the file path of the currently active tab."""
+        try:
+            # Get the current tab index
+            current_index = self.mda_file.tabWidget.currentIndex()
+            logger.debug(f"getCurrentFilePath - Current tab index: {current_index}")
+
+            if current_index >= 0:
+                # Get the file path directly from the tab manager using the current tab
+                # We can get this from the current file tableview's data
+                current_tableview = self.currentFileTableview()
+                if current_tableview and hasattr(current_tableview, "data"):
+                    file_info = current_tableview.data().get("fileInfo", {})
+                    file_path = file_info.get("filePath")
+                    if file_path:
+                        logger.debug(
+                            f"getCurrentFilePath - Found file path: {file_path}"
+                        )
+                        return file_path
+
+                logger.debug(
+                    "getCurrentFilePath - No file path found in current tableview"
+                )
+            return None
+        except Exception as e:
+            logger.debug(f"getCurrentFilePath - Error: {e}")
+            return None
+
+    def clearOtherTabs(self, keep_file_path):
+        """Clear all tabs except the one with the specified file path."""
+        try:
+            logger.debug(
+                f"clearOtherTabs - Starting with keep_file_path: {keep_file_path}"
+            )
+
+            # Get all file paths from tab manager
+            all_file_paths = list(self.mda_file.tabManager.tabs().keys())
+            logger.debug(f"clearOtherTabs - All file paths: {all_file_paths}")
+
+            tabs_to_remove = []
+
+            # Find file paths to remove (all except keep_file_path)
+            for file_path in all_file_paths:
+                if file_path != keep_file_path:
+                    tabs_to_remove.append(file_path)
+                    logger.debug(f"clearOtherTabs - Will remove: {file_path}")
+
+            logger.debug(f"clearOtherTabs - Tabs to remove: {tabs_to_remove}")
+
+            # Remove tabs using the tab manager
+            for file_path in tabs_to_remove:
+                logger.debug(f"clearOtherTabs - Removing tab: {file_path}")
+                self.mda_file.tabManager.removeTab(file_path)
+
+            logger.debug(
+                f"clearOtherTabs - Removed {len(tabs_to_remove)} tabs, kept: {keep_file_path}"
+            )
+        except Exception as e:
+            logger.debug(f"clearOtherTabs - Error: {e}")
